@@ -24,10 +24,11 @@ except ImportError:
 
 
 CLASS_NAMES = ['Pre-explosion', 'SNIa-norm', 'SNIbc', 'SNII', 'SNIa-91bg', 'SNIa-x', 'point-Ia', 'Kilonova', 'SLSN-I',
-               'PISN', 'ILOT', 'CART', 'TDE']
+               'PISN', 'ILOT', 'CART', 'TDE', 'AGN']
 CLASS_COLOR = {'Pre-explosion': 'grey', 'SNIa-norm': 'tab:green', 'SNIbc': 'tab:orange', 'SNII': 'tab:blue',
                'SNIa-91bg': 'tab:red', 'SNIa-x': 'tab:purple', 'point-Ia': 'tab:brown', 'Kilonova': '#aaffc3',
-               'SLSN-I': 'tab:olive', 'PISN': 'tab:cyan', 'ILOT': '#FF1493', 'CART': 'navy', 'TDE': 'tab:pink'}
+               'SLSN-I': 'tab:olive', 'PISN': 'tab:cyan', 'ILOT': '#FF1493', 'CART': 'navy', 'TDE': 'tab:pink',
+               'AGN': 'bisque'}
 PB_COLOR = {'u': 'tab:blue', 'g': 'tab:blue', 'r': 'tab:orange', 'i': 'm', 'z': 'k', 'Y': 'y'}
 PB_MARKER = {'g': 'o', 'r': 's'}
 PB_ALPHA = {'g': 0.3, 'r': 1.}
@@ -35,16 +36,12 @@ SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
 
 
 class Classify(object):
-    def __init__(self, light_curves, known_redshift=True, model_filepath='', passbands=('g', 'r'), bcut=True, zcut=None, graph=None, model=None):
+    def __init__(self, known_redshift=True, model_filepath='', passbands=('g', 'r'),
+                 bcut=False, zcut=None, graph=None, model=None):
         """ Takes a list of photometric information and classifies light curves as a function of time
 
         Parameters
         ----------
-        light_curves : list
-            Is a list of tuples. Each tuple contains the light curve information of a transient object in the form
-            (mjd, flux, fluxerr, passband, zeropoint, photflag, ra, dec, objid, redshift, mwebv).
-            Here, mjd, flux, fluxerr, passband, zeropoint, and photflag are arrays.
-            ra, dec, objid, redshift, and mwebv are floats
         known_redshift : bool
             Different model to be used if redshift is not known.
         model_filepath : str
@@ -55,7 +52,7 @@ class Classify(object):
             Cut on galactic latitude.
             Do not set unless you know what you are doing.
         zcut : float or None
-            Remove resdhifts above this value.
+            Remove redshifts above this value.
             Do not set unless you know what you are doing.
         graph : tensorflow graph
             Do not set unless you know what you are doing.
@@ -67,11 +64,11 @@ class Classify(object):
             This would have been created with keras' load_model function e.g. model = load_model('keras_model.hdf5')
 
         """
-        self.light_curves = light_curves
         self.known_redshift = known_redshift
         self.passbands = passbands
         self.bcut = bcut
         self.zcut = zcut
+        self.class_names = CLASS_NAMES
 
         if self.known_redshift:
             self.contextual_info = (0,)
@@ -95,51 +92,122 @@ class Classify(object):
         else:
             self.model = load_model(self.model_filepath)
 
-    def process_light_curves(self):
-        processed_lightcurves = read_multiple_light_curves(self.light_curves, known_redshift=self.known_redshift,
+    def process_light_curves(self, light_curves):
+        processed_lightcurves = read_multiple_light_curves(light_curves, known_redshift=self.known_redshift,
                                                            training_set_parameters=None)
         prepareinputarrays = PrepareInputArrays(self.passbands, self.contextual_info, self.bcut, self.zcut)
-        X, orig_lc, timesX, objids_list = prepareinputarrays.prepare_input_arrays(processed_lightcurves)
+        X, orig_lc, timesX, objids_list, trigger_mjds = prepareinputarrays.prepare_input_arrays(processed_lightcurves)
 
-        return X, orig_lc, timesX, objids_list
+        # # REMOVE CORRECTION FACTOR IF NOT USED
+        # correction_factor = np.load('astrorapid/correction_factor.npy')
+        # for i, pb in enumerate(self.passbands):
+        #     X[:, :, i] = X[:, :, i] / correction_factor[i]
 
-    def get_predictions(self, return_argmax=False):
-        """ Return the classifcation accuracies as a function of time for each class
+        return X, orig_lc, timesX, objids_list, trigger_mjds
+
+    def get_predictions(self, light_curves, return_predictions_at_obstime=False, return_objids=False):
+        """ Return the classification accuracies as a function of time for each class
 
         Parameters
         ----------
-        return_argmax: bool
-            return the maximum timestep before the rest of the light curve input image is zeroed out.
+        light_curves : list
+            Is a list of tuples. Each tuple contains the light curve information of a transient object in the form
+            (mjd, flux, fluxerr, passband, photflag, ra, dec, objid, redshift, mwebv).
+            Here, mjd, flux, fluxerr, passband, and photflag are arrays.
+            And ra, dec, objid, redshift, and mwebv are floats.
+                mjd: list of Modified Julian Dates of light curve
+
+                flux: list of fluxes at each mjd
+
+                fluxerr: list of flux errors
+
+                passband: list of strings indicating the passband. 'r' or 'g' for r-band or g-band observations.
+
+                photflag: list of flags identifying whether the observation is a detection (4096), non-detection (0),
+                or the first detection (6144).
+
+                ra: Right Ascension (float value).
+
+                dec: Declination (float value).
+
+                objid: Object Identifier (String).
+
+                redshift: Cosmological redshift of object (float). Set to NoneType if redshift is unknown.
+
+                mwebv: Milky way extinction.
+
+        return_predictions_at_obstime: bool
+            Return the predictions at the observation times instead of at the 50 interpolated timesteps.
+        return_objids : bool, optional
+            If True, also return the object IDs (objids) in the same order as the returned predictions.
 
         Returns
         -------
-        self.y_predict: array
+        y_predict: array
             Classification probability vector at each time step for each object.
-            Array of shape (s, 50, 13) is returned.
+            Array of shape (s, n, m) is returned.
             Where s is the number of obejcts that are classified,
-            50 is the number of times steps, and 13 is the number of classes.
-        argmax: array
-            Array of shape (s x 1) is returned. Where s is the number of obejcts that are classified.
-            Each element indicates the last index of self.y_predict before the rest of the array is zeroed and
-            probabilities are not meaningful.
-
+            n is the number of times steps, and m is the number of classes.
+        time_steps: array
+            MJD time steps corresponding to the timesteps of the y_predict array.
+        objids : array, optional
+            The object ids (objids) that were input into light_curves are returned in the same order as y_predict.
+            Only provided if return_objids is True.
         """
-        self.X, self.orig_lc, self.timesX, self.objids = self.process_light_curves()
+
+        # Do error checks
+        assert isinstance(light_curves, (list, np.ndarray))
+        for light_curve in light_curves:
+            assert len(light_curve) == 10
+            mjd, flux, fluxerr, passband, photflag, ra, dec, objid, redshift, mwebv = light_curve
+            _lcshape = len(mjd)
+            assert _lcshape == len(flux)
+            assert _lcshape == len(fluxerr)
+            assert _lcshape == len(passband)
+            assert _lcshape == len(photflag)
+
+        self.X, self.orig_lc, self.timesX, self.objids, self.trigger_mjds = self.process_light_curves(light_curves)
+        nobjects = len(self.objids)
+
+        if nobjects == 0:
+            print("No objects to classify. These may have been removed from the chosen selection cuts")
+            return None, None
 
         if self.graph is not None:
-
             with self.graph.as_default():
                 self.y_predict = self.model.predict(self.X)
         else:
             self.y_predict = self.model.predict(self.X)
 
-        if return_argmax:
-            argmax = self.timesX.argmax(axis=1) + 1
-            return self.y_predict, argmax
+        argmax = self.timesX.argmax(axis=1) + 1
 
-        return self.y_predict
+        if return_predictions_at_obstime:
+            (s, n, m) = self.y_predict.shape  # (s, n, m) = (num light curves, num timesteps, num classes)
+            y_predict = []
+            time_steps = []
+            for idx in range(s):
+                obs_time = []
+                for pb in self.passbands:
+                    if pb in self.orig_lc[idx]:
+                        obs_time.append(self.orig_lc[idx][pb]['time'].values)
+                obs_time = np.array(obs_time)
+                obs_time = np.sort(obs_time[~np.isnan(obs_time)])
+                y_predict_at_obstime = []
+                for classnum, classname in enumerate(CLASS_NAMES):
+                    y_predict_at_obstime.append(np.interp(obs_time, self.timesX[idx][:argmax[idx]], self.y_predict[idx][:, classnum][:argmax[idx]]))
+                y_predict.append(np.array(y_predict_at_obstime).T)
+                time_steps.append(obs_time + self.trigger_mjds[idx])
+        else:
+            y_predict = [self.y_predict[i][:argmax[i]] for i in range(nobjects)]
+            time_steps = [self.timesX[i][:argmax[i]] + self.trigger_mjds[i] for i in range(nobjects)]
 
-    def plot_light_curves_and_classifications(self, indexes_to_plot=None, savename=None, step=True, use_interp_flux=False):
+        if return_objids:
+            return y_predict, time_steps, self.objids
+
+        return y_predict, time_steps
+
+    def plot_light_curves_and_classifications(self, indexes_to_plot=None, step=True, use_interp_flux=False, figdir='.',
+                                              plot_matrix_input=False, light_curves=None):
         """
         Plot light curve (top panel) and classifications (bottom panel) vs time.
 
@@ -149,10 +217,20 @@ class Classify(object):
             The indexes listed in the tuple will be plotted according to the order of the input light curves.
             E.g. (0, 1, 3, 5) will plot the zeroth, first, third and fifth light curves and classifications.
             If None or True, then all light curves will be plotted
-        savename : str
-            Filename to save plot.
         step : bool
-            Plot step function along data points instead of interpolating classifications between data
+            Plot step function along data points instead of interpolating classifications between data.
+        use_interp_flux : bool
+            Use all 50 timesteps when plotting classification probabilities rather than just at the timesteps with data.
+        figdir : str
+            Directory to save figure.
+        plot_matrix_input : bool
+            Plots the interpolated light curve passed into the neural network on top of the observations.
+        light_curves : list
+            This argument is only required if the get_predictions() method has not been run.
+            Is a list of tuples. Each tuple contains the light curve information of a transient object in the form
+            (mjd, flux, fluxerr, passband, photflag, ra, dec, objid, redshift, mwebv).
+            Here, mjd, flux, fluxerr, passband, and photflag are arrays.
+            ra, dec, objid, redshift, and mwebv are floats
 
         """
 
@@ -164,8 +242,12 @@ class Classify(object):
             indexes_to_plot = np.arange(len(self.y_predict))
 
         if not hasattr(self, 'y_predict'):
-            self.get_predictions()
+            self.get_predictions(light_curves)
 
+        if plot_matrix_input:
+            alpha_observations = 0.2
+        else:
+            alpha_observations = 1.
         for idx in indexes_to_plot:
             argmax = self.timesX[idx].argmax() + 1
             fig, (ax1, ax2) = plt.subplots(nrows=2, ncols=1, figsize=(13, 15),
@@ -173,28 +255,31 @@ class Classify(object):
             # ax1.axvline(x=0, color='k', linestyle='-', linewidth=1)
             # ax2.axvline(x=0, color='k', linestyle='-', linewidth=1)
 
-            for pb in self.passbands:
-                if pb in  self.orig_lc[idx].keys():
+            for pbidx, pb in enumerate(self.passbands):
+                if pb in self.orig_lc[idx].keys():
                     ax1.errorbar(self.orig_lc[idx][pb]['time'], self.orig_lc[idx][pb]['flux'],
                                  yerr=self.orig_lc[idx][pb]['fluxErr'], fmt=PB_MARKER[pb], label=pb,
-                                 c=PB_COLOR[pb], lw=3, markersize=10)
+                                 c=PB_COLOR[pb], lw=3, markersize=10, alpha=alpha_observations)
+                if plot_matrix_input:
+                    ax1.plot(self.timesX[idx][:argmax], self.X[idx][:, pbidx][:argmax], c=PB_COLOR[pb], lw=3)
 
-            new_t = np.array(list(self.orig_lc[idx]['g']['time']) + list(self.orig_lc[idx]['r']['time']))
+            new_t = np.array([self.orig_lc[idx][pb]['time'].values for pb in self.passbands]).flatten()
             new_t = np.sort(new_t[~np.isnan(new_t)])
-            if not use_interp_flux:
+            if not use_interp_flux and not plot_matrix_input:
                 new_y_predict = []
                 for classnum, classname in enumerate(CLASS_NAMES):
                     new_y_predict.append(np.interp(new_t, self.timesX[idx][:argmax], self.y_predict[idx][:, classnum][:argmax]))
 
             for classnum, classname in enumerate(CLASS_NAMES):
-                if not use_interp_flux:
+                if use_interp_flux or plot_matrix_input:
+                    ax2.plot(self.timesX[idx][:argmax], self.y_predict[idx][:, classnum][:argmax], '-', label=classname,
+                             color=CLASS_COLOR[classname], linewidth=3)
+                else:
                     if step:
                         ax2.step(new_t, new_y_predict[classnum], '-', label=classname, color=CLASS_COLOR[classname], linewidth=3, where='post')
                     else:
                         ax2.plot(new_t, new_y_predict[classnum], '-', label=classname, color=CLASS_COLOR[classname], linewidth=3,)
-                else:
-                    ax2.plot(self.timesX[idx][:argmax], self.y_predict[idx][:, classnum][:argmax], '-', label=classname,
-                         color=CLASS_COLOR[classname], linewidth=3)
+
             ax1.legend(frameon=True, fontsize=33)#, loc='lower right')
             ax2.legend(frameon=True, fontsize=21.5)#, loc='center right')  # , ncol=2)
             ax2.set_xlabel("Days since trigger (rest frame)")  # , fontsize=18)
@@ -209,15 +294,14 @@ class Classify(object):
             ax2.yaxis.set_major_locator(MaxNLocator(nbins=6, prune='upper'))  # added
             plt.tight_layout()
             fig.subplots_adjust(hspace=0)
-            if savename is None:
-                savename = 'classification_vs_time_{}{}{}'.format(self.objids[idx], '_step' if step else '', '_no_interp' if not use_interp_flux else '')
-            plt.savefig("{}.pdf".format(savename))
+            savename = 'classification_vs_time_{}{}{}.pdf'.format(self.objids[idx], '_step' if step else '', '_no_interp' if not use_interp_flux else '')
+            plt.savefig(os.path.join(figdir, savename))
             # plt.savefig("{}.png".format(savename))
             plt.close()
 
         return self.orig_lc, self.timesX, self.y_predict
 
-    def plot_classification_animation(self, indexes_to_plot=None):
+    def plot_classification_animation(self, indexes_to_plot=None, figdir='.', light_curves=None):
         """ Plot light curve (top panel) and classifications (bottom panel) vs time as an mp4 animation.
 
         Parameters
@@ -226,6 +310,14 @@ class Classify(object):
             The indexes listed in the tuple will be plotted according to the order of the input light curves.
             E.g. (0, 1, 3, 5) will plot the zeroth, first, third and fifth light curves and classifications.
             If None or True, then all light curves will be plotted
+        figdir : str
+            Directory to save figure.
+        light_curves : list
+            This argument is only required if the get_predictions() method has not been run.
+            Is a list of tuples. Each tuple contains the light curve information of a transient object in the form
+            (mjd, flux, fluxerr, passband, photflag, ra, dec, objid, redshift, mwebv).
+            Here, mjd, flux, fluxerr, passband, and photflag are arrays.
+            ra, dec, objid, redshift, and mwebv are floats
 
         """
 
@@ -237,10 +329,10 @@ class Classify(object):
             indexes_to_plot = np.arange(len(self.y_predict))
 
         if not hasattr(self, 'y_predict'):
-            self.get_predictions()
+            self.get_predictions(light_curves)
 
         for idx in indexes_to_plot:
-            new_t = np.array(list(self.orig_lc[idx]['g']['time']) + list(self.orig_lc[idx]['r']['time']))
+            new_t = np.array([self.orig_lc[idx][pb]['time'].values for pb in self.passbands]).flatten()
             all_flux = list(self.orig_lc[idx]['g']['flux']) + list(self.orig_lc[idx]['r']['flux'])
 
             argmax = self.timesX[idx].argmax() + 1
@@ -286,9 +378,10 @@ class Classify(object):
                 ax2.legend(by_label2.values(), by_label2.keys(), frameon=False, fontsize=21.5, loc='center right')
 
             ani = animation.FuncAnimation(fig, animate, frames=50, repeat=True)
-            ani.save(os.path.join('classification_vs_time_{}.mp4'.format(self.objids[idx])), writer=writer)
+            savename = os.path.join(figdir,'classification_vs_time_{}.mp4'.format(self.objids[idx]))
+            ani.save(savename, writer=writer)
 
-    def plot_classification_animation_step(self, indexes_to_plot=None, savename=None):
+    def plot_classification_animation_step(self, indexes_to_plot=None, figdir='.', light_curves=None):
         """
         Plot light curve (top panel) and classifications (bottom panel) vs time as an mp4 animation
         as step function.
@@ -299,8 +392,14 @@ class Classify(object):
             The indexes listed in the tuple will be plotted according to the order of the input light curves.
             E.g. (0, 1, 3, 5) will plot the zeroth, first, third and fifth light curves and classifications.
             If None or True, then all light curves will be plotted
-        savename : str
-            Filename to save plot.
+        figdir : str
+            Directory to save figure.
+        light_curves : list
+            This argument is only required if the get_predictions() method has not been run.
+            Is a list of tuples. Each tuple contains the light curve information of a transient object in the form
+            (mjd, flux, fluxerr, passband, photflag, ra, dec, objid, redshift, mwebv).
+            Here, mjd, flux, fluxerr, passband, and photflag are arrays.
+            ra, dec, objid, redshift, and mwebv are floats
 
         """
 
@@ -315,7 +414,7 @@ class Classify(object):
             self.get_predictions()
 
         for idx in indexes_to_plot:
-            new_t = np.array(list(self.orig_lc[idx]['g']['time']) + list(self.orig_lc[idx]['r']['time']))
+            new_t = np.array([self.orig_lc[idx][pb]['time'].values for pb in self.passbands]).flatten()
             new_t = np.sort(new_t[~np.isnan(new_t)])
             new_y_predict = []
             all_flux = list(self.orig_lc[idx]['g']['flux']) + list(self.orig_lc[idx]['r']['flux'])
@@ -379,7 +478,6 @@ class Classify(object):
                 ax2.legend(by_label2.values(), by_label2.keys(), frameon=False, fontsize=21.5, loc='center right')
 
             ani = animation.FuncAnimation(fig, animate, frames=len(new_t), repeat=True)
-            if savename is None:
-                savename = 'classification_vs_time_{}_step'.format(self.objids[idx])
-            ani.save("{}.mp4".format(savename), writer=writer)
+            savename = os.path.join(figdir, 'classification_vs_time_{}_step.mp4'.format(self.objids[idx]))
+            ani.save(savename, writer=writer)
 
