@@ -6,6 +6,7 @@ import pickle
 import multiprocessing as mp
 import numpy as np
 import pandas as pd
+import itertools
 from scipy.interpolate import interp1d
 
 from astrorapid import helpers
@@ -52,10 +53,14 @@ class PrepareArrays(object):
             print("Redshift cut. z = {}".format(redshift))
             deleterows.append(i)
             deleted = True
-        elif class_num is not None and variables_cut is True and class_num in [50, 70, 80, 81, 83, 84, 90, 91, 92, 93]:
+        elif class_num is not None and variables_cut is True and class_num in [70, 80, 81, 83, 84, 90, 91, 92, 93]:
             print("Not including variable models", class_num)
             deleterows.append(i)
             deleted = True
+        elif class_num in [50]:
+            print("Deleting unused kilonova model")
+            deleterows.append(i)
+            deleted = True
 
         return deleterows, deleted
 
@@ -84,6 +89,7 @@ class PrepareArrays(object):
             len_t = len(tinterp)
             if len_t > self.nobs:
                 tinterp = tinterp[:-(len_t - self.nobs)]
+                len_t = len(tinterp)
         return tinterp, len_t
 
     def update_X(self, X, i, data, tinterp, len_t, objid, contextual_info, otherinfo):
@@ -96,15 +102,22 @@ class PrepareArrays(object):
             data.replace([np.inf, -np.inf], np.nan)
 
             # Get data
-            time = data[pb]['time'][0:self.nobs].dropna()
-            flux = data[pb]['flux'][0:self.nobs].dropna()
-            fluxerr = data[pb]['fluxErr'][0:self.nobs].dropna()
-            photflag = data[pb]['photflag'][0:self.nobs].dropna()
+            time = data[pb]['time'][0:self.nobs].dropna().values
+            flux = data[pb]['flux'][0:self.nobs].dropna().values
+            fluxerr = data[pb]['fluxErr'][0:self.nobs].dropna().values
+            photflag = data[pb]['photflag'][0:self.nobs].dropna().values
+
+            # Mask out times outside of mintime and maxtime
+            timemask = (time > self.mintime) & (time < self.maxtime)
+            time = time[timemask]
+            flux = flux[timemask]
+            fluxerr = fluxerr[timemask]
+            photflag = photflag[timemask]
 
             n = len(flux)  # Get vector length (could be less than nobs)
 
             if n > 1:
-                if flux.values[-1] > flux.values[-2]:  # If last values are increasing, then set fill_values to zero
+                if flux[-1] > flux[-2]:  # If last values are increasing, then set fill_values to zero
                     f = interp1d(time, flux, kind='linear', bounds_error=False, fill_value=0.)
                 else:
                     f = interp1d(time, flux, kind='linear', bounds_error=False,
@@ -134,51 +147,6 @@ class PrepareArrays(object):
 
         return X
 
-    def update_X_nointerp(self, X, i, data, tinterp, len_t, objid, contextual_info, otherinfo):
-        pb_wave = {'u': 357, 'g': 477, 'r': 621, 'i': 754, 'z': 871, 'Y': 1004}
-
-        # Drop infinite values
-        data.replace([np.inf, -np.inf], np.nan)
-
-        # Get data
-        for pb in self.passbands:
-            data[pb, 'passband'] = pb_wave[pb]
-        data_stackedpb = pd.concat([data[pb].dropna() for pb in self.passbands])
-        data_stackedpb = data_stackedpb.sort_values('time')
-        data_stackedpb['deltat'] = np.insert(np.diff(data_stackedpb['time']), 0, 0)
-
-        len_t = data_stackedpb.shape[0]
-        if len_t >= self.nobs:
-            data_stackedpb = data_stackedpb[data_stackedpb['time'] >= self.mintime]
-            len_t = data_stackedpb.shape[0]
-            if len_t >= self.nobs:
-                tinterp = data_stackedpb[data_stackedpb['time'] <= self.maxtime]
-                len_t = data_stackedpb.shape[0]
-
-        X[i][0][0:len_t] = data_stackedpb['deltat']
-        X[i][1][0:len_t] = data_stackedpb['flux']
-        X[i][2][0:len_t] = data_stackedpb['passband']
-        # X[i][3][0:len_t] = data_stackedpb['fluxErr']
-        lr = 2  # last row
-
-        # Add contextual information
-        for jj, c_idx in enumerate(contextual_info, 1):
-            try:
-                X[i][lr + jj][0:len_t] = otherinfo[c_idx] * np.ones(len_t)
-            except Exception as e:
-                X[i][lr + jj][0:len_t] = otherinfo[c_idx].values[0] * np.ones(len_t)
-
-
-
-        # Add contextual information
-        for jj, c_idx in enumerate(contextual_info, 1):
-            try:
-                X[i][j + jj][0:len_t] = otherinfo[c_idx] * np.ones(len_t)
-            except Exception as e:
-                X[i][j + jj][0:len_t] = otherinfo[c_idx].values[0] * np.ones(len_t)
-
-        return X
-
 
 class PrepareInputArrays(PrepareArrays):
     def __init__(self, passbands=('g', 'r'), contextual_info=(0,), bcut=True, zcut=None):
@@ -196,6 +164,7 @@ class PrepareInputArrays(PrepareArrays):
         objids_list = []
         orig_lc = []
         deleterows = []
+        trigger_mjds = []
 
         for i, (objid, data) in enumerate(lightcurves.items()):
             print("Preparing light curve {} of {}".format(i, nobjects))
@@ -213,17 +182,22 @@ class PrepareInputArrays(PrepareArrays):
             timesX[i][0:len_t] = tinterp
             orig_lc.append(data)
             objids_list.append(objid)
+            trigger_mjds.append(trigger_mjd)
             X = self.update_X(X, i, data, tinterp, len_t, objid, self.contextual_info, otherinfo)
 
+        deleterows = np.array(deleterows)
+        X = np.delete(X, deleterows, axis=0)
+        timesX = np.delete(timesX, deleterows, axis=0)
+
         # Correct shape for keras is (N_objects, N_timesteps, N_passbands) (where N_timesteps is lookback time)
         X = X.swapaxes(2, 1)
 
-        return X, orig_lc, timesX, objids_list
+        return X, orig_lc, timesX, objids_list, trigger_mjds
 
 
 class PrepareTrainingSetArrays(PrepareArrays):
     def __init__(self, passbands=('g', 'r'), contextual_info=(0,), reread=False, aggregate_classes=False, bcut=True,
-                 zcut=None, variablescut=False, nchunks=10000):
+                 zcut=None, variablescut=False, nchunks=10000, training_set_dir='training_set_files'):
         PrepareArrays.__init__(self, passbands, contextual_info)
         self.passbands = passbands
         self.contextual_info = contextual_info
@@ -234,7 +208,7 @@ class PrepareTrainingSetArrays(PrepareArrays):
         self.variablescut = variablescut
         self.nchunks = nchunks
         self.agg_map = helpers.aggregate_sntypes()
-        self.training_set_dir = 'training_set_files'
+        self.training_set_dir = training_set_dir
         if not os.path.exists(self.training_set_dir):
             os.makedirs(self.training_set_dir)
 
@@ -268,7 +242,7 @@ class PrepareTrainingSetArrays(PrepareArrays):
                                                                               self.contextual_info,
                                                                               os.path.basename(fpath_saved_lc),
                                                                               self.zcut, self.bcut, self.variablescut))
-
+        print(savepath)
         if self.reread is True or not os.path.isfile(savepath):
             objids, self.fpath = self.get_saved_light_curves_from_database(fpath_saved_lc)
             nobjects = len(objids)
@@ -276,7 +250,8 @@ class PrepareTrainingSetArrays(PrepareArrays):
             # Store data labels (y) and 'r' band data (X). Use memory mapping because input file is very large.
             labels = np.zeros(shape=nobjects, dtype=np.uint16)
             y = np.zeros(shape=(nobjects, self.nobs), dtype=np.uint16)
-            X = np.memmap(os.path.join(self.training_set_dir, 'X_lc_data.dat'), dtype=np.float16, mode='w+', shape=(nobjects, self.nfeatures, self.nobs))
+            X = np.memmap(os.path.join(self.training_set_dir, 'X_lc_data.dat'), dtype=np.float32, mode='w+',
+                          shape=(nobjects, self.nfeatures, self.nobs))
             X[:] = np.zeros(shape=(nobjects, self.nfeatures, self.nobs))
             timesX = np.zeros(shape=(nobjects, self.nobs))
             objids_list = []
@@ -296,6 +271,7 @@ class PrepareTrainingSetArrays(PrepareArrays):
             sum_deleterows = 0
             startidx = 0
             num_outputs = len(outputs)
+            print('combining results...')
             for i, output in enumerate(outputs):
                 labels_part, y_part, X_part, timesX_part, objids_list_part, orig_lc_part, num_deleterows_part, num_objects_part = output
                 endidx = startidx + num_objects_part
@@ -307,7 +283,6 @@ class PrepareTrainingSetArrays(PrepareArrays):
                 orig_lc.extend(orig_lc_part)
                 startidx += num_objects_part
                 sum_deleterows += num_deleterows_part
-                print('combining results...', i, num_outputs)
 
             deleterows = np.array(np.arange(nobjects - sum_deleterows, nobjects))
             X = np.delete(X, deleterows, axis=0)
@@ -430,6 +405,49 @@ class PrepareTrainingSetArrays(PrepareArrays):
         # Correct shape for keras is (N_objects, N_timesteps, N_passbands) (where N_timesteps is lookback time)
         X = X.swapaxes(2, 1)
 
+        # #NORMALISE
+        # X = X.copy()
+        # for i in range(len(X)):
+        #     for pbidx in range(2):
+        #         minX = X[i, :, pbidx].min(axis=0)
+        #         maxX = X[i, :, pbidx].max(axis=0)
+        #         X[i, :, pbidx] = (X[i, :, pbidx] - minX) / (maxX - minX)
+        #         # if (maxX - minX) != 0:
+        #         #     mask.append(i)
+        #         #     break
+        # finitemask = ~np.any(np.any(~np.isfinite(X), axis=1), axis=1)
+        # X = X[finitemask]
+        # y = y[finitemask]
+        # timesX = timesX[finitemask]
+        # objids_list = objids_list[finitemask]
+        # orig_lc = list(itertools.compress(orig_lc, finitemask))
+        # labels = labels[finitemask]
+
+        newX = np.zeros(X.shape)
+        newy = np.zeros(y.shape)
+        lenX = len(X)
+        for i in range(lenX):
+            if i % 1000 == 0:
+                print(f"new {i} of {lenX}")
+            mask = timesX[i] > 0
+            nmask = sum(mask)
+            newX[i][:nmask] = X[i][mask]
+            newy[i][:nmask] = y[i][mask]
+
+        print("Concatenating")
+        X = np.concatenate((X, newX))
+        y = np.concatenate((y, newy))
+        labels = np.concatenate((labels, labels))
+        timesX = np.concatenate((timesX, timesX))
+        orig_lc = orig_lc * 2
+        objids_list = np.concatenate((objids_list, objids_list))
+        print("Shuffling")
+        from sklearn.utils import shuffle
+        X, y, labels, timesX, orig_lc, objids_list = shuffle(X, y, labels, timesX, orig_lc, objids_list)
+        print("Done shuffling")
+
+
+
         X_train, X_test, y_train, y_test, labels_train, labels_test, timesX_train, timesX_test, orig_lc_train, orig_lc_test, objids_train, objids_test = train_test_split(
             X, y, labels, timesX, orig_lc, objids_list, train_size=0.60,
             shuffle=False, random_state=42)
@@ -483,7 +501,7 @@ class PrepareTrainingSetArrays(PrepareArrays):
 
             # Make cuts
             deleterows, deleted = self.make_cuts(data, i, deleterows, b, redshift, class_num=model, bcut=self.bcut,
-                                                 zcut=self.zcut, variables_cut=self.variablescut, pre_trigger=True)
+                                                 zcut=self.zcut, variables_cut=self.variablescut, pre_trigger=False)
             if deleted:
                 continue
 
@@ -506,3 +524,4 @@ class PrepareTrainingSetArrays(PrepareArrays):
         num_objects = X.shape[0]
 
         return labels, y, X, timesX, objids_list, orig_lc, count_deleterows, num_objects
+
